@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 # G-language GAE is free software; you can redistribute it and/or
@@ -221,7 +221,7 @@ sub db_save{
 	my $level = shift || 0;
 	my $id = shift || '';
 
-	if(defined %$hash){
+	if(%$hash){
 	    foreach my $key (keys %$hash) {
 		if($level == $depth - 1){
 		    my $sqlline = '';
@@ -261,7 +261,7 @@ sub nested_hash_depth {
 	my $hash = shift;
 	my $level = shift || 0;
 	
-	if(defined %$hash){
+	if(%$hash){
 	    foreach my $key (keys %$hash) {
 		recursive_search($hash->{$key}, $level + 1);
 	    }
@@ -286,7 +286,7 @@ sub nested_hash_keys{
 	my $hash = shift;
 	my $depth = shift || 0;
 	
-	if(defined %$hash){
+	if(%$hash){
 	    foreach my $key (keys %$hash) {
 		if($depth == $level - 1){
 		    $keys{$key} ++;
