@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: G::IO.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -47,7 +47,7 @@ use File::Basename;
 use File::ShareDir ':ALL';
 use autouse 'File::Temp'=>qw(tempfile);
 
-our $VERSION = '1.8.12';
+our $VERSION = '1.8.13';
 
 #::::::::::::::::::::::::::::::
 #          Perldoc
@@ -201,16 +201,17 @@ sub new {
 
     }elsif ($format eq 'usa'){
 	my ($dbname, $entryname) = split(/:/, $filename, 2);
-	msg_error("Retrieving sequence $entryname from " . uc($dbname) . "...\n");
 
-	if($dbname eq 'sw' || $dbname eq 'uniprot'){
-	    $dbname = 'swiss';
+	if($dbname eq 'swiss' || $dbname eq 'sw' || $dbname eq 'uniprot' || $dbname eq 'swissprot' || $dbname eq 'swall'){
+	    $dbname = 'uniprot';
 	}
 
 	if($dbname eq 'refseq'){
 	    $dbname = 'genbank';
 	}
 
+	msg_error("Retrieving sequence $entryname from " . uc($dbname) . "...\n");
+
 	if($dbname eq 'genbank' || $dbname eq 'embl'){
 	    my $path = _sdb_path() . $entryname . '.' . $dbname;
 	    mirror('http://togows.dbcls.jp/entry/' . $dbname . '/'. $entryname, $path);
@@ -220,6 +221,15 @@ sub new {
 		$this = new G::IO($path);
 	    }
 	    croak('Sequence not found in database at http://togows.dbcls.jp/entry/' . $dbname . '/'. $entryname) unless(length($this->{SEQ}));
+	}elsif($dbname eq 'uniprot'){
+	    my $path = _sdb_path() . $entryname . '.' . $dbname;
+	    mirror('http://togows.dbcls.jp/entry/' . $dbname . '/'. $entryname, $path);
+	    if($no_msg){
+		$this = new G::IO($path, 'no msg', 'swiss');
+	    }else{
+		$this = new G::IO($path, 'swiss');
+	    }
+	    croak('Sequence not found in database at http://togows.dbcls.jp/entry/' . $dbname . '/'. $entryname) unless(length($this->{SEQ}));
 	}else{
 	    my $bp;
 	    
@@ -318,7 +328,7 @@ sub loaded_msg {
 	         Arakawa K. et al. (2008) Genes, Genomes and Genomics.
 
  	      License: GNU General Public License
-	      Copyright (C) 2001-2010 G-language Project
+	      Copyright (C) 2001-2011 G-language Project
 	      Institute for Advanced Biosciences, Keio University, JAPAN 
 
 	     __/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/
