@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: GenBankO.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -220,6 +220,10 @@ sub make_gb {
 		    }
 		}
 	    }else{
+		if($key eq 'transl_except'){
+		    $gb->{$feat}->{$key} = '(' . $gb->{$feat}->{$key} . ')';
+		}
+
 		my $str = '/' . $key . '="' . $gb->{$feat}->{$key} . '"';
 		$lng = length($str);
 		if($lng >= 58 ){
