@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: G::IO::Handler.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -135,6 +135,7 @@ sub clone {
     my $new = sdb_load($tmpfile);
 
     _set_sdb_path($sdbPath);
+    unlink($tmpfile);
 
     return $new;
 }
@@ -828,6 +829,7 @@ sub pos2gene {
          Given a CDS ID, returns the CDS sequence, or the exon sequence
          If introns are present.
          'complement' is properly parsed, and introns are spliced out.
+         Codon start position is used when codon_start key is available.
 
          eg. $gb->get_geneseq('recA'); # returns the 'recA' gene sequence or exon.
 
@@ -853,7 +855,8 @@ sub get_geneseq {
 	$geneseq = &complement($geneseq);
     }
 
-    return $geneseq;
+    my $codon_start = $this->{$object}->{codon_start} || 1;
+    return substr($geneseq, $codon_start - 1);
 }
 
 =head2 $gb->get_intron()
@@ -1272,7 +1275,7 @@ sub next_feature{
     my $i = $this->{$feature}->{feature};
     $i ++;
 
-    while(defined(%{$this->{"FEATURE$i"}})){
+    while(%{$this->{"FEATURE$i"}}){
 	my $feat = "FEATURE$i";
 	$i ++;
 
@@ -1314,7 +1317,7 @@ sub previous_feature{
     my $i = $this->{$feature}->{feature} if ($feature =~ /^CDS/);
     $i --;
 
-    while(defined(%{$this->{"FEATURE$i"}})){
+    while(%{$this->{"FEATURE$i"}}){
 	my $feat = "FEATURE$i";
 	$i --;
 
@@ -1384,7 +1387,7 @@ sub intergenic {
 
     set_intergenic($this) unless(defined($this->{INTER1}));
 
-    while(defined(%{$this->{"INTER" . ($i + 1)}})){
+    while(%{$this->{"INTER" . ($i + 1)}}){
 	$i ++;
 
 	if ($opt ne 'all'){
@@ -1711,7 +1714,7 @@ sub _interpret_format {
     if ($ref =~ /Bio::Seq/){
 	return 'bioperl';
     }elsif ($filename =~ /^(.*?):(.*)/){
-	unless(lc($1) =~ /(swiss|sw|uniprot|genbank|genpept|embl|refseq)/){
+	unless(lc($1) =~ /(swall|swiss|sw|uniprot|genbank|genpept|embl|refseq)/){
 	    croak("Unsupported database name for USA. Supported databases are\n" .
 		"swiss, sw, uniprot, genbank, genpept, embl, or refseq\n");
 	}
@@ -1832,6 +1835,53 @@ sub bioperl {
 }
 
 
+
+=head2 $gb->set_generationtime()
+
+   Name: $gb->set_generationtime()   -   add generation time data for bacterial genomes
+
+   Description:
+         This method adds generation time data for bacterial genomes,
+         based on the data in Reference 1.
+
+         $gb->{FEATURE0}->{generationTime} is the generation time (hour), and
+         $gb->{feature0}->{optimumGrowthTemperature} is the optimum growth temperature in Celsius.
+
+  References: 
+    1. Vieira-Silva S and Rocha RPC (2010) "The systemic imprint of growth and its uses in 
+       ecological (meta)genomics", PLoS Genetics, 6(1):e1000808
+
+  REST: 
+      http://rest.g-language.org/NC_000913/set_generationtime
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  HISTORY:
+     20110224-01 initial posting
+
+=cut
+
+
+sub set_generationtime {
+    my $gb = shift;
+
+    my $filename = dist_file('g-language', 'data/generationtime.dat');
+    for my $line (readFile($filename, 1)){
+	next unless($line =~ $gb->{LOCUS}->{id});
+
+	my @F = split(/\t/, $line);
+	$gb->{FEATURE0}->{'generationTime'} = $F[3];
+	$gb->{FEATURE0}->{'optimumGrowthTemperature'} = $F[2];
+    }
+
+    unless(length($gb->{FEATURE0}->{'generationTime'})){
+	msg_error("No generation time data for this species.\n\n");
+    }
+
+    return 1;
+}
+
 =head2 $gb->set_essentiality()
 
    Name: $gb->set_essentiality()   -   add gene essentiality data for E.coli genome 
@@ -1843,6 +1893,10 @@ sub bioperl {
 
          $gb->{$cds}->{essentiality} is 1 for essential, -1 for non-essential genes.
 
+  References:
+    1. Hashimoto M et al. (2005) "Cell size and nucleoid organization of engineered 
+       Escherichia coli cells with a reduced genome", Mol Microbiol, 55(1):137-149
+
   REST: 
       http://rest.g-language.org/NC_000913/set_essentiality
 
