@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Align.pm,v 1.2 2002/07/30 17:40:56 gaou Exp $
@@ -95,19 +95,17 @@ sub alignment{
     opt_default(Gap=>-5, Match=>10, Miss=>-7, output=>'stdout', filename=>'alignment.txt');
     my @args = opt_get(@_);
 
-    my $gb   = opt_as_gb(shift @args);
-    my $seq1 = \$gb->{SEQ};
-       $gb   = opt_as_gb(shift @args);
-    my $seq2 = \$gb->{SEQ};
+    my $seq1 = shift @args;
+    my $seq2 = shift @args;
     my %opt  = opt_val();
     my %align;
     my ($H, $V, $D, $NDIR) = (0, 1, 2, 3);
     my @score_path_matrix;
 
-    $$seq1 =~ tr/ \n\t[0-9]//d;
-    $$seq2 =~ tr/ \n\t[0-9]//d;
-    my $i = length($$seq1);
-    my $j = length($$seq2); 
+    $seq1 =~ tr/ \n\t[0-9]//d;
+    $seq2 =~ tr/ \n\t[0-9]//d;
+    my $i = length($seq1);
+    my $j = length($seq2); 
 
     local *find_max_score = sub{
 	my ($i, $j, $seq1, $seq2, $Gap_Penalty, $Match, $Miss) = @_;
@@ -126,7 +124,7 @@ sub alignment{
 	    }else{
 		$score_tmp[$V] = find_max_score($i - 1, $j, $seq1, $seq2, $Gap_Penalty, $Match, $Miss) + $Gap_Penalty;
 		$score_tmp[$H] = find_max_score($i, $j - 1, $seq1, $seq2, $Gap_Penalty, $Match, $Miss) + $Gap_Penalty;
-		if(substr($$seq1, $i - 1, 1) eq substr($$seq2, $j - 1, 1)){
+		if(substr($seq1, $i - 1, 1) eq substr($seq2, $j - 1, 1)){
 		    $score_tmp[$D] = find_max_score($i - 1, $j - 1, $seq1, $seq2, $Gap_Penalty, $Match, $Miss) + $Match;
 		}else{
 		    $score_tmp[$D] = find_max_score($i - 1, $j - 1, $seq1, $seq2, $Gap_Penalty, $Match, $Miss) + $Miss;
@@ -154,19 +152,19 @@ sub alignment{
     while($i > 0 | $j > 0){
 	if($score_path_matrix[$i][$j]{direction} == $H){
 	    $align{seq1} = '-' . $align{seq1};
-	    $align{seq2} = substr($$seq2, $j - 1, 1) . $align{seq2};
+	    $align{seq2} = substr($seq2, $j - 1, 1) . $align{seq2};
 	    $align{hit}  = '-' . $align{hit};
 	    $j--;
 	}
 	if($score_path_matrix[$i][$j]{direction} == $V){
-	    $align{seq1} = substr($$seq1, $i - 1, 1) . $align{seq1};
+	    $align{seq1} = substr($seq1, $i - 1, 1) . $align{seq1};
 	    $align{seq2} = '-' . $align{seq2};
 	    $align{hit}  = '-' . $align{hit};
 	    $i--;
 	}
 	if($score_path_matrix[$i][$j]{direction} == $D){
-	    my $tmp1 = substr($$seq1, $i - 1, 1);
-	    my $tmp2 = substr($$seq2, $j - 1, 1);
+	    my $tmp1 = substr($seq1, $i - 1, 1);
+	    my $tmp2 = substr($seq2, $j - 1, 1);
 	    $align{seq1} = $tmp1 . $align{seq1};
 	    $align{seq2} = $tmp2 . $align{seq2};
 	    if($tmp1 eq $tmp2){
@@ -185,7 +183,7 @@ sub alignment{
     my $gap_num2 = $align{seq2} =~ tr/-/-/;
 
     my $result;
-    $result .= sprintf("Seq1: %d bp   Seq2: %d bp\n", length($$seq1), length($$seq2));
+    $result .= sprintf("Seq1: %d bp   Seq2: %d bp\n", length($seq1), length($seq2));
     $result .= sprintf("Score: %d   Match: %d   Miss: %d\n", $align{score}, $hit_num, $miss_num);
     $result .= sprintf("Gap in seq1: %d   Gap in seq2: %d\n", $gap_num1, $gap_num2);
     $result .= sprintf("Match score: %d   Miss score: %d   Gap Penalty: %d\n\n", $opt{'Match'}, $opt{'Miss'}, $opt{'Gap'});
