@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Align.pm,v 1.2 2002/07/30 17:40:56 gaou Exp $
@@ -37,6 +37,8 @@ our @EXPORT = qw(
 		 calc_pI
 		 amino_info
 		 peptide_mass
+		 one2three
+		 three2one
 );
 
 #__DATA__
@@ -275,16 +277,16 @@ sub hydropathy{
     return %hydropathy;
 }
 
-=head2 G::Seq::AminoAcid::one2three
+=head2 one2three
 
-  Name: G::Seq::AminoAcid::one2three   -   converts one letter amino acid code to three letter code
+  Name: one2three   -   converts one letter amino acid code to three letter code
 
   Description:
     This method converts one letter amino acid code to three letter code.
 
   Usage: 
-    $amino = G::Seq::AminoAcid::one2three($aminocid);
-    eg. G::Seq::AminoAcid::one2three('A'); # returns 'Ala';
+    $amino = one2three($aminocid);
+    eg. one2three('A'); # returns 'Ala';
 
   Options:
     none
@@ -293,12 +295,14 @@ sub hydropathy{
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20101105-01 added support for amino acid sequence
     20010404-01 initial posting
 
 =cut
 
 sub one2three{
     my $key = shift;
+    my $ret = '';
 
     my %one2three = ( 
 		      A=>"Ala", B=>"Asx", C=>"Cys", D=>"Asp", E=>"Glu", F=>"Phe",
@@ -308,20 +312,24 @@ sub one2three{
 		      U=>"Sec", '/'=>"TER"
 		      );
 
-    return $one2three{$key} if (length $key);
+    for my $char (split(//, uc($key))){
+	$ret .= $one2three{$char};
+    }
+
+    return $ret if (length $key);
     return %one2three;
 }
 
-=head2 G::Seq::AminoAcid::three2one
+=head2 three2one
 
-  Name: G::Seq::AminoAcid::three2one   -   converts three letter amino acid code to one letter code
+  Name: three2one   -   converts three letter amino acid code to one letter code
 
   Description:
     This method converts three letter amino acid code to one letter code.
 
   Usage: 
-    $amino = G::Seq::AminoAcid::three2one($aminocid);
-    eg. G::Seq::AminoAcid::three2one('Ala'); # returns 'A';
+    $amino = three2one($aminocid);
+    eg. three2one('Ala'); # returns 'A';
 
  Options:
     none
@@ -330,15 +338,20 @@ sub one2three{
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20101105-01 added support for amino acid sequence
     20010404-01 initial posting
 
 =cut
 
 sub three2one{
     my $key = shift;
+    my $ret = '';
     my %three2one = reverse one2three();
 
-    return $three2one{$key} if (length $key);
+    for(my $i = 0; $i < length($key); $i ++){
+	$ret .= $three2one{substr($key, $i, 3)};
+    }
+    return $ret if (length $key);
     return reverse one2three();
 }
 
