@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: GCskew.pm,v 1.3 2002/08/16 15:03:36 gaou Exp $
@@ -27,11 +27,16 @@
 package G::Seq::GCskew;
 
 use SubOpt;
+use Rcmd;
 use G::Messenger;
 use G::Tools::Graph;
 use G::Tools::Statistics;
 use G::Seq::Primitive;
+use G::Seq::Codon;
+use G::Seq::Util;
 use autouse 'Algorithm::Numerical::Shuffle'=>qw(shuffle);
+use File::ShareDir ':ALL';
+use File::Temp;
 
 use strict;
 use base qw(Exporter);
@@ -43,6 +48,7 @@ our @EXPORT = qw(
 	     gcsi
 	     cum_gcskew
 	     genomicskew
+	     geneskew
 	     gcwin
 	     leading_strand
 	     query_strand
@@ -52,10 +58,14 @@ our @EXPORT = qw(
 	     genes_from_ori
 	     rep_ori_ter
 	     dist_in_cc
+	     lda_bias
+	     B1
+	     B2
+	     delta_gcskew
 );
 
 
-__DATA__
+#__DATA__
 
 #:::::::::::::::::::::::::::::::::
 #       Perldoc
@@ -240,37 +250,67 @@ sub find_ori_ter {
  Name: rep_ori_ter   -   get the positions of replication origin and terminus
 
  Description:
-    This method returns the documented replicational origin and terminus.
-    Currently positions are available in the following genomes:
-     [1] Escherichia coli K12, Bacillus subtilis, Haemophilus influenzae
-     [2] Salmonella typhi
+    This method returns the positions of replication origin and terminus
+    in bacterial genomes by several means. 
 
-    If "rep_origin" feature is available, center position within this feature
-    is used for origin. 
+    1. Use of databases
+      By default, rep_ori_ter() tries to retrieve the position of replication
+      origin in DoriC (Reference 1) database, and the position of replication
+      terminus from the supplemental data provided in Reference 2.
 
-    If no documentation is available, the origin and terminus is predicted using
-    find_ori_ter().
+      If the position of origin cannot be found in the database, but "rep_origin" 
+      feature is available, center position within this feature is used for origin. 
+
+    2. Oriloc
+      Using -oriloc=>1 option, you can predict the replication origin and 
+      terminus using the popular Oriloc program developed by Lobry et al. 
+      available as part of the SeqinR package (Reference 3).
+
+      Installation of SeqinR package is required to use this feature.
+      Type: install.packages('seqinr', dep=T)
+      in R statistics package to install SeqinR.
+
+    3. Setting custom origin and terminus
+      Use -ori=>$position1, -ter=>$position2 to set your own positions.
+
+    4. Use GC skew shift-point
+       If the positions of origin or terminus cannot be found in the databases,
+       rep_ori_ter() automatically calls find_ori_ter() method to predict the 
+       positions using GC skew shift-points at one-base-pair resolution.
 
     After calling this function, positions of origin and terminus are stored 
     as follows:
       $genome->{FEATURE0}->{origin}
       $genome->{FEATURE0}->{terminus}
 
+    You can always use -clear=>1 option to disable the use of above cache.
+
  Usage: 
     ($ori, $ter) = rep_ori_ter($genome);
 
  References:
-   1. Freeman JM et al. (1998) "Patterns of Genome Organization in Bacteria", 
-      Science, 279(5358):1827a
-   2. Parkhill J et al. (2001) "Complete genome sequence of a multiple drug 
-      resistant Salmonella enterica serovar Typhi CT18", Nature, 413(6858):848-852 
+   1. Gao F and Zhang CT (2007) "DoriC: a database of oriC regions in bacterial
+      genomes", Bioinformatics, 23(14):1866-1867
+   2. Kono N et al. (2011) "Comprehensive prediction of chromosome dimer resolution 
+      sites in bacterial genomes", BMC Genomics, 12(1):19
+   3. Frank AC and Lobry JR (2000) "Oriloc: prediction of replication boundaries 
+      in unannotated bacterial chromosomes", Bioinformatics, 16(6):560-561
 
  Options:
-   none
+   -oriloc            set to 1 to use Oriloc for prediction (default: 0)
+   -gcskew            set to 1 to use GC skew shift-point for prediction (default: 0)
+   -dif-threshold     distance between the GC skew shift point and predicted dif site
+                      expressed as the percentage of genome size, used as a threshold
+                      to retrieve dif sequence from the database (default: 5)
+   -dbonly            set to 1 to only use values available in databases and 
+                      to suppress prediction
+   -clear             set to 1 to disable the use of cache
 
  Author: Kazuharu Arakawa
 
  History:
+    20110223-01 major revision. added support for DoriC and dif databases,
+                Oriloc (SeqinR), and an option -gcskew
     20080428-01 added support for "rep_origin" feature
     20011030-01 initial posting
 
@@ -278,45 +318,111 @@ sub find_ori_ter {
 
 
 sub rep_ori_ter {
+    opt_default('dif-threshold'=>5, 'gcskew'=>0, 'clear'=>0, 'oriloc'=>0, 'dbonly'=>0);
     my @args = opt_get(@_);
-    my $gb = opt_as_gb(shift @args);
+    my $gb   = opt_as_gb(shift @args);
+    my %opt  = opt_val();
+    my $id   = $gb->{LOCUS}->{id};
     my ($ori, $ter);
-    my $id = $gb->{LOCUS}->{id};
 
+    if($opt{'clear'}){
+	delete($gb->{FEATURE0}->{origin});
+	delete($gb->{FEATURE0}->{terminus});
+    }
 
-    if(length $gb->{FEATURE0}->{terminus}){
-	$ori = $gb->{FEATURE0}->{origin};
-	$ter = $gb->{FEATURE0}->{terminus};
-    }else{
-
-	if ($id eq 'U00096' || $id eq 'NC_000913'){
-	    $ori = 3923500 - 1; 
-	    $ter = 1588800 - 1; 
-	}elsif ($id eq 'AL009126' || $id eq 'NC_000964'){
-	    $ori = 1 - 1;
-	    $ter = 2017000 - 1;
-	}elsif ($id eq 'L42023' || $id eq 'NC_000907'){
-	    $ori = 603000 - 1;
-	    $ter = 1518000 - 1;
-	}elsif ($id eq 'AL513382' || $id eq 'NC_003198'){
-	    $ori = 3765000 - 1;
-	    $ter = 1437000 - 1;
-	}else{
-	    msg_error('Using find_ori_ter() to predict...' . "\n");
-	    ($ori, $ter) = &G::Seq::GCskew::find_ori_ter($gb, -output=>"/dev/null");
-	}
-
-	my $interface = msg_ask_interface();
-	msg_interface('NULL');
-	my $cds = ($gb->find(-type=>'rep_origin'))[0];
-	if($cds){
-	    $ori = int(($gb->{$cds}->{end} + $gb->{$cds}->{start})/2) - 1;
-	    msg_error("rep_origin feature found. Using $ori as origin.\n");
-	}
-	msg_interface($interface);
+    if(length($opt{'ori'}) || length($opt{'ter'})){
+	$ori = $opt{'ori'} if (length($opt{'ori'}));
+	$ter = $opt{'ter'} if (length($opt{'ter'}));
+	msg_error("Using ori: $ori and ter: $ter\n");
 
 	$gb->{FEATURE0}->{origin} = $ori;
 	$gb->{FEATURE0}->{terminus} = $ter;
+    }elsif(length $gb->{FEATURE0}->{terminus}){
+	$ori = $gb->{FEATURE0}->{origin};
+	$ter = $gb->{FEATURE0}->{terminus};
+    }elsif($opt{'oriloc'}){
+	msg_error("Using Oriloc in SeqinR package to predict...\n");
+	my $gbk = '/tmp/' . time() . rand() . '.gbk';
+	my $rcmd = new Rcmd();
+	$rcmd->set_mode('silent');
+	$gb->output($gbk);
+
+	($ori, $ter) = $rcmd->exec(qq|
+				   library(seqinr);
+				   x = oriloc(gbk='$gbk')
+				   c(x[which(x[,5]==max(x[,5])),2] * 1000, x[which(x[,5]==min(x[,5])),2] * 1000)
+				   |);
+
+	unlink($gbk);
+
+	msg_error("\nPredicted ori: $ori and ter: $ter\n");
+
+	$gb->{FEATURE0}->{origin} = $ori;
+	$gb->{FEATURE0}->{terminus} = $ter;
+    }else{
+	unless($opt{'gcskew'}){
+	    my @oris;
+	    for my $line (readFile("http://tubic.tju.edu.cn/doric/query.php?selfield=oa&term=$id")){
+		if($line =~ /(information.php\?ac\=ORI\d+)/){
+		    my $flag = 0;
+		    for my $ll (readFile("http://tubic.tju.edu.cn/doric/$1")){
+			$flag = 1 if ($ll =~ /The location of oriC region/);
+
+			if($flag && $ll =~ /(\d+)\.\.(\d+) nt/){
+			    push(@oris, $1, $2);
+			    last;
+			}
+		    }
+		}
+	    }
+
+	    if(scalar(@oris)){
+		$ori = $oris[int(scalar(@oris)/2)];
+		msg_error("oriC found in DoriC. Using $ori as origin.\n");
+	    }
+
+	    my $filename = dist_file('g-language', 'data/dif.tab');
+
+	    my $data = {};
+	    open(FILE, $filename);
+	    while(<FILE>){
+		chomp;
+		my @line = split(/\t/, $_);
+		next unless($line[0] =~ /^NC_/);
+		
+		$data->{$line[0]}->{'delta'} = $line[4];
+		$data->{$line[0]}->{'dif'}   = $line[5];
+	    }
+	    close(FILE);
+	    
+	    if(length($data->{$id}) && $data->{$id}->{'delta'} <= $opt{'dif-threshold'} * 3.6){
+		$ter = $data->{$id}->{'dif'};
+		msg_error("dif sequence found in database. Using $ter as terminus.\n");
+	    }
+	    
+	    unless($ori){
+		my $interface = msg_ask_interface();
+		msg_interface('NULL');
+		my $cds = ($gb->find(-type=>'rep_origin'))[0];
+		if($cds){
+		    $ori = int(($gb->{$cds}->{end} + $gb->{$cds}->{start})/2) - 1;
+		    msg_error("rep_origin feature found. Using $ori as origin.\n");
+		}
+		msg_interface($interface);
+	    }
+	}
+
+	unless($opt{'dbonly'}){
+	    if(length($ori) < 1 || length($ter) < 1){
+		msg_error('Using GC skew shift-point [find_ori_ter()] to predict...' . "\n");
+		my ($ori2, $ter2) = &G::Seq::GCskew::find_ori_ter($gb, -output=>"/dev/null");
+		$ori = $ori2 unless(length($ori));
+		$ter = $ter2 unless(length($ter));
+	    }
+	    
+	    $gb->{FEATURE0}->{origin} = $ori;
+	    $gb->{FEATURE0}->{terminus} = $ter;
+	}
     }
 
     return ($ori, $ter);
@@ -663,9 +769,121 @@ sub cum_gcskew {
 
 
 
+=head2 geneskew
+
+ Name: geneskew   -   calculate the gene strand bias of the given genome
+
+ Description:
+   This program calculates and graphs the strand bias of genes (or the GC skew within them).
+   By default, this program visualizes the gene strand preference (1 for direct, -1 for complement strand),
+   but by specifying -base option  option, GC/AT/Purine/Keto skews of the coding regions
+   or more specifically in the GC3 (third codon position) with -gc3 option can be calculated.
+
+ Usage: 
+    array @geneskew = geneskew(G instance);
+
+ Options:
+   -window      window size to observe (default: 10000)
+   -slide       window slide size (default: same as window size)
+   -cumulative  1 to calculate cumulative skew (default: 0)
+   -base        'gc', 'at', 'purine', or 'keto' for observing GC/AT/Purine/Keto skews (default: none)
+   -gc3         use only the third codon positions.
+   -output      f for file output in directory "data", 
+                g for graph output in directory "graph",
+                show for graph output and display (default: "show")
+   -filename    output filename (default: "gcskew.png" for -output=>"g",
+                                          "gcskew.csv" for -output=>"f")
+
+ Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+ History:
+   200110216-01 initial posting
+
+=cut
+
+
+
+sub geneskew {
+    &opt_default(output=>"show", filename=>"geneskew.png", cumulative=>0);
+    my @args       = opt_get(@_);
+    my $gb         = opt_as_gb(shift @args);
+    my %opts       = opt_val();
+    my @skew       = ();
+    my @location   = ();
+
+    if($opts{'gc3'} && length($opts{'base'}) < 1){
+	$opts{'base'} = 'gc';
+    }
+
+    for my $cds ($gb->cds()){
+	push(@location, $gb->{$cds}->{start});
+
+	if($opts{'base'}){
+	    my $geneseq = $gb->get_geneseq($cds);
+
+	    if($opts{'gc3'}){
+		my $seq = '';
+		my $i = 0;
+		for($i = 2; $i < length $geneseq; $i += 3){
+		    $seq .= substr($geneseq, $i, 1);
+		}
+		$geneseq = $seq;
+	    }
+	    $geneseq = complement($geneseq) if ($gb->{$cds}->{direction} eq 'complement');
+
+	    my $type = '-' . $opts{'base'};
+	    my @gcskew = gcskew($geneseq, -window=>length($geneseq), -output=>"null", $type=>1);
+	    push(@skew, $gcskew[0]);
+
+	}else{
+	    push(@skew, $gb->{$cds}->{direction} eq 'direct' ? 1:-1);
+	}
+    }
+    @skew = cumulative(@skew) if ($opts{'cumulative'});
+
+    my $cumulative = $opts{'cumulative'} == 1 ? 'Cumulative ' : '';
+    my $type = '';
+    if(length($opts{'base'})){
+	$type = length($opts{'base'}) == 2 ? uc($opts{'base'}) : ucfirst($opts{'base'});
+	$type .= ' ';
+    }
+    my $title = $cumulative . 'gene ' . $type . 'skew';
+    $title .= ' (GC3)' if ($opts{'gc3'});
+
+    if ($opts{'output'} eq 'g' || $opts{'output'} eq 'show'){
+	mkdir ("graph", 0777);
+
+	grapher(
+		 \@location,\@skew,
+		 -x=>"bp", -y=>$title,
+		 -filename=>$opts{'filename'},
+		 -title=>$title,
+		 -style=>"lines", -type=>"columns",
+		 -output=>$opts{'output'}
+		);
+
+    }elsif ($opts{'output'} eq 'f'){
+	$opts{'filename'} =~ s/\.png$/\.csv/;
+	my $j = 0;
+	mkdir ("data", 0777);
+
+	open(OUT, ">data/" . $opts{'filename'});
+	print OUT "location,$title\n";
+	for ($j = 0; $j <= scalar(@skew); $j++){
+	    print OUT $location[$j], ",", $skew[$j], "\n";
+	}
+	close(OUT);
+    }
+    
+    return @skew;
+}
+
+
+
 =head2 genomicskew
 
- Name: genomicskew   -   calculate the GC skew in different  regions of the given genome
+ Name: genomicskew   -   calculate the GC skew in different regions of the given genome
 
  Description:
    This program graphs the GC skew for the whole genome, coding regions,
@@ -1315,4 +1533,349 @@ sub dist_in_cc{
 
 
 
+=head2 lda_bias
+
+  Name: lda_bias   -   calculate strand bias of bacterial genome using linear discriminant analysis (LDA)
+
+  Description:
+    This method calculate strand bias of bacterial genome using linear discriminant 
+    analysis (LDA), as proposed in Reference 1. The basic idea is to use composition
+    data of genes to train and predict the strand of genes residing either on the 
+    leading or the lagging strand. For computational efficiency, this method trans
+    and predicts the strands at putative replication origin as reported by the 
+    rep_ori_ter() method. This usually results in maximum predictability of LDA
+    within bacterial genomes.
+
+    Data to use for LDA can be chosen from "base", "codonbase", "codon", and "amino",
+    with -variable option.
+
+    Installation of R statistics package is required.
+
+ Usage: 
+    $bias = lda_bias($genome);
+
+ References:
+   1. Rocha EPC et al. (1999) "Universal replication biases in bacteria",
+      Molecular Microbiology, 32(1):11-16
+
+  Options:
+   -variable        data to use for LDA. Either "base", "codonbase", "codon", or "amino".
+                    (default: codon)
+   -coefficients    show LDA coefficients (default: 0)
+
+  Author: Kazuharu Arakawa
+
+  History:
+    20110223-01 initial posting
+
+=cut
+
+
+
+sub lda_bias{
+    &opt_default('variable'=>'codon', 'coefficients'=>0);
+    my @args = opt_get(@_);
+    my $gb   = opt_as_gb(shift @args);
+    my %data = opt_val();
+
+    my $fh = File::Temp->new();
+    my $file = $fh->filename;
+
+    if($data{'variable'} eq 'base'){
+	print $fh "strand,a,t,g,c\n";
+	for my $cds ($gb->cds()){
+	    my $seq = $gb->get_geneseq($cds);
+	    print $fh join(',', query_strand($gb, $cds), map{$_/length($seq)} seqinfo($seq, -output=>"null")), "\n";
+	}
+
+    }elsif($data{'variable'} eq 'codonbase'){
+	print $fh "strand,1a,1t,1g,1c,2a,2t,2g,2c,3a,3t,3g,3c\n";
+
+	for my $cds ($gb->cds()){
+	    my @pos;
+	    my $seq = $gb->get_geneseq($cds);
+	    for (my $i = 0; $i < length($seq); $i ++){
+		$pos[$i % 3] .= substr($seq, $i, 1);
+	    }
+
+	    print $fh join(',', query_strand($gb, $cds), 
+			   (map{$_/length($pos[0])} seqinfo($pos[0], -output=>"null")), 
+			   (map{$_/length($pos[1])} seqinfo($pos[1], -output=>"null")), 
+			   (map{$_/length($pos[2])} seqinfo($pos[2], -output=>"null"))
+			   ), "\n";
+	}
+
+    }elsif($data{'variable'} eq 'codon'){
+	my $genomic = codon_compiler($gb, -output=>"null", -data=>'R0');
+	for (keys %$genomic){
+	    delete($$genomic{$_}) if ($$genomic{$_} < scalar($gb->cds())/2);
+	}
+	print $fh join(',', 'strand', sort keys %$genomic), "\n";
+
+	for my $cds ($gb->cds()){
+	    my @res;
+	    my $codon = codon_compiler($gb, -output=>"null", -data=>'R1', -id=>$cds);
+	    for my $key (sort keys %$genomic){
+		push(@res, $$codon{$key} || 0);
+	    }
+	    print $fh join(',', query_strand($gb, $cds), @res), "\n";
+	}
+
+    }elsif($data{'variable'} eq 'amino'){
+	my $genomic = codon_compiler($gb, -output=>"null", -data=>'A0');
+	for (keys %$genomic){
+	    delete($$genomic{$_}) if ($$genomic{$_} < scalar($gb->cds())/2);
+	}
+	print $fh join(',', 'strand', sort keys %$genomic), "\n";
+
+	for my $cds ($gb->cds()){
+	    my @res;
+	    my $codon = codon_compiler($gb, -output=>"null", -data=>'A1', -id=>$cds);
+	    for my $key (sort keys %$genomic){
+		push(@res, $$codon{$key} || 0);
+	    }
+	    print $fh join(',', query_strand($gb, $cds), @res), "\n";
+	}
+
+    }else{
+	die("lda_bias: Bad parameter for -variable option. Choose from 'base', 'codonbase', 'codon', and 'amino'.");
+    }
+
+#    my $g = $data{'coefficients'} ? 'g' : '';
+    my $r = new Rcmd;
+    $r->set_mode('silent');
+
+    my $ret =  $r->exec(
+			qq|
+			X = read.csv('$file', header=TRUE);
+			library(MASS);
+			g = lda(strand~.,data=X);
+			g;
+			v = predict(g,X);
+			p = sum(v\$class == X[,1])/length(X[,1]);
+			p;
+			|
+			);
+
+    msg_error(readFile($r->{log})) if($data{'coefficients'});
+    return $ret;
+}
+
+
+
+
+
+
+=head2 B1
+
+  Name: B1   -   calculate strand bias of bacterial genome using B1 index
+
+  Description:
+    This method calculate strand bias of bacterial genome using B1 index,
+    first proposed by Lobry and Sueoka in Reference 1, and further extended by 
+    Rocha et al. in Reference 2. Basic idea of B1 index is to calculate the 
+    distance between the two strands, when the leading and lagging strands
+    are plotted in a coordinate system with axes representing G/(G+C) and A/(A+T),
+    using the third codon position of genes. This index measures the degree of 
+    replication-induced bias from Chargaff's second parity rule.
+
+    Rocha et al. modified B1 index to only use >fourfold degenerate codons,
+    and to use T/(A+T) in place of A/(A+T).
+
+ Usage: 
+    $bias = B1($genome);
+
+ References:
+   1. Lobry JR and Sueoka N (2002) "Asymmetric directional mutation pressures in bacteria",
+      Genome Biology, 3(10):0058
+   2. Rocha EPC et al. (2006) "Similar compositional biases are caused by very different
+      mutational effects", Genome Research, 16:1537-1547
+
+  Options:
+   -method         choose the method of 'lobry' or 'rocha' (default: 'rocha')
+
+  Author: Kazuharu Arakawa
+
+  History:
+    20110223-01 initial posting
+
+=cut
+
+
+
+sub B1{
+    &opt_default('method'=>'rocha');
+    my @args   = opt_get(@_);
+    my $gb     = opt_as_gb(shift @args);
+    my %option = opt_val();
+
+    my $data = {};
+
+    for my $cds ($gb->cds()){
+	my $seq    = $gb->get_geneseq($cds);
+	my $strand = query_strand($gb, $cds);
+
+	for (my $i = 0; $i < length($seq); $i +=3){
+	    if($option{'method'} eq 'lobry'){
+		$data->{$strand}->{substr($seq, $i+2, 1)}++;
+	    }else{
+		my $codon = substr($seq, $i, 2);
+		if($codon eq 'gc' || $codon eq 'gg' || 
+		   $codon eq 'cc' || $codon eq 'ac' || $codon eq 'gt'  
+		   || $codon eq 'ct' || $codon eq 'tc' || $codon eq 'cg'
+		   ){
+
+		    $data->{$strand}->{substr($seq, $i+2, 1)}++;		    
+		}
+	    }
+	}
+    }
+
+    my $nuc = $option{'method'} eq 'lobry' ? 'a' : 't';
+    
+    return sqrt( 
+		   ($data->{'leading'}->{'g'} /($data->{'leading'}->{'g'}+$data->{'leading'}->{'c'}) 
+		  - $data->{'lagging'}->{'g'} /($data->{'lagging'}->{'g'}+$data->{'lagging'}->{'c'})) ** 2 +
+		   ($data->{'leading'}->{$nuc}/($data->{'leading'}->{'a'}+$data->{'leading'}->{'t'}) 
+		  - $data->{'lagging'}->{$nuc}/($data->{'lagging'}->{'a'}+$data->{'lagging'}->{'t'})) ** 2 );
+}
+
+
+=head2 B2
+
+  Name: B2   -   calculate strand bias of bacterial genome using B2 index
+
+  Description:
+    This method calculate strand bias of bacterial genome using B2 index,
+    proposed by Lobry and Sueoka in Reference 1. Basic idea of B2 index is 
+    to calculate the distance from neutral parity state (0.5, 0.5), when the bias of
+    the coding regions is plotted in a coordinate system with axes representing 
+    G/(G+C) and A/(A+T), using the third codon position of genes. This index measures 
+    the degree of transcription- and translation-associated effects of bias from 
+    Chargaff's second parity rule.
+
+ Usage: 
+    $bias = B2($genome);
+
+ References:
+   1. Lobry JR and Sueoka N (2002) "Asymmetric directional mutation pressures in bacteria",
+      Genome Biology, 3(10):0058
+
+  Options:
+    None.
+
+  Author: Kazuharu Arakawa
+
+  History:
+    20110223-01 initial posting
+
+=cut
+
+
+sub B2{
+    &opt_default();
+    my @args   = opt_get(@_);
+    my $gb     = opt_as_gb(shift @args);
+    my %option = opt_val();
+    my %data;
+
+    for my $cds ($gb->cds()){
+	my $seq    = $gb->get_geneseq($cds);
+
+	for (my $i = 0; $i < length($seq); $i +=3){
+	    $data{substr($seq, $i+2, 1)}++;
+	}
+    }
+
+    return sqrt( ($data{'g'}/($data{'g'}+$data{'c'}) - 0.5) ** 2 + ($data{'a'}/($data{'a'}+$data{'t'}) - 0.5) ** 2 );
+}
+
+
+
+
+
+=head2 delta_gcskew
+
+  Name: delta_gcskew   -   calculate strand bias of bacterial genome using delta GC skew index
+
+  Description:
+    This method calculate strand bias of bacterial genome using delta GC skew index,
+    first proposed by Rocha et al in Reference 1, and further extended  
+    in Reference 2. Basic idea of delta GC skew index is to calculate the 
+    difference of GC skew in coding regions residing in leading and lagging strands.
+    Reference 1 calculates delta GC skew index using the third codon position of genes,
+    and Reference 2  modified to only use >fourfold degenerate codons.
+
+ Usage: 
+    $bias = delta_gcskew($genome);
+
+ References:
+   1. Rocha EPC et al. (2001) "Ongoing Evolution of Strand Composition in Bacterial Genomes",
+      Molecular Biology and Evolution, 18(9):1789-1799
+   2. Rocha EPC et al. (2006) "Similar compositional biases are caused by very different
+      mutational effects", Genome Research, 16:1537-1547
+
+  Options:
+   -method      choose the nucleotides to use 'degenerate', 'gc3', or 'all' (default: 'degenerate')
+   -at          1 when observing AT skew instead of GC skew (default: 0)
+   -purine      1 when observing purine (AG/TC) skew (default: 0)
+   -keto        1 when observing keto (TG/AC) skew (default: 0)
+
+  Author: Kazuharu Arakawa
+
+  History:
+    20110223-01 initial posting
+
+=cut
+
+
+
+sub delta_gcskew{
+    &opt_default('method'=>'degenerate');
+    my @args   = opt_get(@_);
+    my $gb     = opt_as_gb(shift @args);
+    my %option = opt_val();
+
+    my %data;
+
+    for my $cds ($gb->cds()){
+	my $seq    = $gb->get_geneseq($cds);
+	my $strand = query_strand($gb, $cds);
+
+	if($option{'method'} eq 'all'){
+	    $data{$strand} .= $seq;
+	}elsif($option{'method'} eq 'gc3'){
+	    for (my $i = 0; $i < length($seq); $i +=3){
+		$data{$strand} .= substr($seq, $i+2, 1);
+	    }
+	}else{
+	    for (my $i = 0; $i < length($seq); $i +=3){
+		my $codon = substr($seq, $i, 2);
+		if($codon eq 'gc' || $codon eq 'gg' || 
+		   $codon eq 'cc' || $codon eq 'ac' || $codon eq 'gt'  
+		   || $codon eq 'ct' || $codon eq 'tc' || $codon eq 'cg'
+		   ){
+			$data{$strand} .= substr($seq, $i+2, 1);
+		    }
+	    }
+	}
+    }
+
+    my %send;
+    for my $key (keys %option){
+	$send{'-' . $key} = $option{$key};
+    }
+
+    my @lead = gcskew($data{'leading'}, %send, -output=>"null", -window=>length($data{'leading'}));
+    my @lagg = gcskew($data{'lagging'}, %send, -output=>"null", -window=>length($data{'lagging'}));
+
+    return $lead[0] - $lagg[0];
+}
+
+
+
+
+
+
+
 1;
