@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Util.pm,v 1.1.1.1 2002/04/02 20:25:43 gaou Exp $
@@ -1045,7 +1045,7 @@ sub genome_map {
 	my $flag = 0;
 	my $before = -5000;
 	my $before2 = -10000;
-	while (defined %{$gb->{"CDS$cds"}}){
+	while (%{$gb->{"CDS$cds"}}){
 	    my $cdsstart = $gb->{"CDS$cds"}->{start};
 	    my $cdsend = $gb->{"CDS$cds"}->{end};
 	    my $cdsdir = $gb->{"CDS$cds"}->{direction};
