@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: PatSearch.pm,v 1.3 2002/07/30 17:40:56 gaou Exp $
@@ -245,7 +245,6 @@ sub markov {
 
 
 
-
 =head2 signature
 
   Name: signature   -   calculate oligonucleotide usage (genomic signature)
@@ -253,8 +252,8 @@ sub markov {
   Description:
    This program calculates short oligonucleotide usage (genomic signature),
    defined as the ratio of observed (O) to expected (E) oligonucleotide frequencies.
-   O/E value of dinucleotide CG at position 1..500 will be accessible at 
-   $gb->{1..500}->{signature}->{'cg'}.
+   O/E value of dinucleotide CG will be accessible at 
+   $gb->{signature}->{'cg'}.
     
   Usage: 
    NULL = &signature(G instance);
@@ -264,16 +263,19 @@ sub markov {
                'stdout' for standard output,
                'f' for file output
    -filename   output filename (default: 'signature.csv')
-   -wordlength       word length (default: 2)
-   -start      start position (default: 1)
-   -end        end position (default: 0)
-               0 when using whole genome sequence; i.e. -end=>length($gb->{SEQ});
+   -wordlength word length (default: 2)
+   -bothstrand use direct (0) or both (1) strand (default: 1)
+   -oe         use observed (0) or O/E (1) value (default: 1)
+   -seq        sequence (default: '')
+               '' when using whole genome sequence; i.e. -seq=>$gb->{SEQ}
+   -memo       memo (default: '')
 
   Author: 
    Haruo Suzuki (haruo@g-language.org)
 
   History:
-   20071107 initial posting
+   20101010 initial posting
+   20110222 added -bothstrand, -oe, -seq, and -memo options
 
   Informations stored in the G instance:
     $gb->{"header_signature"}
@@ -287,52 +289,52 @@ sub markov {
 =cut
 
 sub signature {
-    &opt_default(output=>'stdout', filename=>'signature.csv', wordlength=>2, start=>1, end=>0);
+    &opt_default(output=>'stdout', filename=>'signature.csv', wordlength=>2, bothstrand=>1, oe=>1, seq=>'', memo=>'');
     my @args       = opt_get(@_);
     my $gb         = opt_as_gb(shift @args);
     my $output     = opt_val("output");
     my $filename   = opt_val("filename");
     my $wordlength = opt_val("wordlength");
-    my $start      = opt_val("start");
-    my $end        = opt_val("end");
-    my (%mono, %obs, %exp, %oe);
+    my $bothstrand = opt_val("bothstrand");
+    my $oe         = opt_val("oe");
+    my $seq        = opt_val("seq");
+    my $memo       = opt_val("memo");
 
-    $end = length($gb->{SEQ}) unless($end);
-    #my $seq1 = $gb->getseq($start, $end); # Perl coordinates
-    my $seq1 = $gb->get_gbkseq($start, $end); # GenBank coordinates
+    my ($total, %mono, %obs, %exp, %val);
 
-    my $seq2 = reverse($seq1);    
-    $seq2 =~ tr
-	[acgturymkdhbvwsn]
-	[tgcaayrkmhdvbwsn];
+    $seq = $gb->{SEQ} unless($seq);
+    $seq .= ' '.complement($seq) if($bothstrand);
 
-    my $seqb = $seq1.$seq2;
-    $mono{a} = $seqb =~ tr/a/a/;
-    $mono{c} = $seqb =~ tr/c/c/;
-    $mono{g} = $seqb =~ tr/g/g/;
-    $mono{t} = $seqb =~ tr/t/t/;
-    my $total = $seqb =~ tr/acgt/acgt/;
+    $total = $seq =~ tr/acgt/acgt/;
     unless($total){
 	carp("No regular nucleotide found in this sequence.");
 	return;
     }
-
+    $mono{a} = $seq =~ tr/a/a/;
+    $mono{c} = $seq =~ tr/c/c/;
+    $mono{g} = $seq =~ tr/g/g/;
+    $mono{t} = $seq =~ tr/t/t/;
     foreach (keys %mono){ $mono{$_} =  $mono{$_} / $total; }
 
-    for(my $i = 0; $i < length($seq1) - $wordlength + 1; $i ++){
-	$obs{substr($seq1, $i, $wordlength)} ++;
-	$obs{substr($seq2, $i, $wordlength)} ++;
+    for(my $i = 0; $i < length($seq) - $wordlength + 1; $i ++){
+	$obs{substr($seq, $i, $wordlength)} ++;
     }
+
     $total = 0;
     foreach (keys %obs){ delete $obs{$_} if($_ =~ /[^acgt]/); $total += $obs{$_}; }
-    foreach (keys %obs){ $obs{$_} =  $obs{$_} / $total; }
     foreach (keys %obs){
 	$exp{$_} = 1;
 	for(my $i = 0; $i < $wordlength; $i ++){ $exp{$_} *= $mono{substr($_,$i,1)}; }
+	if($oe){ $val{$_} = sprintf("%.3f", ($obs{$_}/$total)/$exp{$_}); } 
+	else { $val{$_} = sprintf("%.3f", $obs{$_}/$total); }
     }
-    foreach (keys %obs){ $oe{$_} = sprintf("%.3f", $obs{$_} / $exp{$_}); }
 
-    $gb->{"$start..$end"}->{signature} = \%oe; # input in the G instance
+    if(0){
+	print "mono=",%mono, "\n";
+	print "mono.total = [$total]\n";
+	print "obs=", %obs, "\n";
+	print "oligo.total = [$total]\n";
+    }
 
     my @allkey = qw(a c g t);
     for(1..($wordlength-1)){
@@ -354,21 +356,18 @@ sub signature {
 		$hashslice{"$key/$key2"} = undef;
 		delete $hashslice{$key};
 		delete $hashslice{$key2};
-		$oe{"$key/$key2"} = sprintf("%.3f", ($oe{$key} + $oe{$key2})/2 );
-		delete $oe{$key};
-		delete $oe{$key2};
+		$val{"$key/$key2"} = sprintf("%.3f", ($val{$key} + $val{$key2})/2 );
+		delete $val{$key};
+		delete $val{$key2};
 	    }
 	}
 	@allkey = sort keys %hashslice;
-	$gb->{"$start..$end"}->{signature} = \%oe; # input in the G instance
     }
     ##### 
-
-    my $GC = $seq1 =~ tr/gcGC/gcGC/; $GC = sprintf "%.1f", 100 * $GC / length($seq1);
     
     if($output =~ /stdout/){
-	msg_send("\n\nstart\tend\tG+C\t"); foreach(@allkey){ msg_send("$_\t"); }
-	msg_send("\n$start\t$end\t$GC\t"); foreach(@allkey){ msg_send("$oe{$_}\t"); }
+	foreach(@allkey){ msg_send("$_\t"); } msg_send("memo\n");
+	foreach(@allkey){ msg_send("$val{$_}\t"); } msg_send("$memo\n");
     }
 
     if($output eq 'f'){
@@ -376,16 +375,17 @@ sub signature {
 	#open(FILE,">>data/$filename");
 	open(my $fh, ">>$filename");
 	unless($gb->{"header_signature"}){
-	    print $fh "start,end,G+C,";
-	    foreach(@allkey){ print $fh "$_,"; }
 	    $gb->{"header_signature"} = 1;
-	    print $fh "\n";
-        }
-	print $fh "$start,$end,$GC,";
-	foreach(@allkey){ print $fh (exists $oe{$_}) ? "$oe{$_}," : "0,"; }
-	print $fh "\n";
+	    foreach(@allkey){ print $fh "$_,"; }
+	    print $fh "memo\n";
+	}
+	foreach(@allkey){ print $fh (exists $val{$_}) ? "$val{$_}," : "0,"; }
+	print $fh "$memo\n";
 	close($fh);
     }
+
+    $gb->{signature} = \%val; # input in the G instance
+    return \%val;
 }
 
 
@@ -418,11 +418,13 @@ sub signature {
    Haruo Suzuki (haruo@g-language.org)
 
   History:
-   20090327 initial posting
+   20101010 initial posting
+   20110222 minor change
 
   References:
    Campbell A et al. (1999) Proc Natl Acad Sci U S A. 96(16):9184-9.
    Suzuki H et al. (2008) Nucleic Acids Res. 36(22):e147.
+   Suzuki H et al. (2010) J Bacteriol. 192(22):6045-55.
 
   Requirements: sub signature()
 
@@ -443,26 +445,25 @@ sub signature_dist {
 #    system('echo -n > query.csv > sbjct.csv');
 
     &signature($query, -output=>"f", -filename=>"/tmp/query$rand.csv", -wordlength=>$wordlength);
-    for(my $i = 1; $i <= length($sbjct->{SEQ}) - $window; $i += $step){ 
-	&signature($sbjct, -output=>"f", -filename=>"/tmp/sbjct$rand.csv", -wordlength=>$wordlength, -start=>$i, -end=>$i+$window); 
+    for(my $i = 1; $i <= length($sbjct->{SEQ}) - $window; $i += $step){
+	my $start = $i;
+	my $end = $i + $window - 1;
+	my $seq = $sbjct->get_gbkseq($start, $end);
+	&signature($sbjct, -output=>"f", -filename=>"/tmp/sbjct$rand.csv", -wordlength=>$wordlength, -seq=>$seq);
     }
-    
+
     my $rcmd = new Rcmd;
     my @result = $rcmd->exec(
 			     qq|
 			     method = "$method"
-			     f = "/tmp/query$rand.csv"
-			     X = read.csv(f)
-			     X = X[,-c(1,2,3,length(X))]
-			     f = "/tmp/sbjct$rand.csv"
-			     Y = read.csv(f)
-			     Y = Y[,-c(1,2,3,ncol(Y))]
+			     X = read.csv("/tmp/query$rand.csv")
+			     X = X[,-ncol(X)]
+			     Y = read.csv("/tmp/sbjct$rand.csv")
+			     Y = Y[,-ncol(Y)]
 			     if(method == "mahalanobis"){ 
                                  library(MASS);
                                  dX = try(mahalanobis(X, apply(Y,2,mean), ginv(var(Y)), inverted=TRUE), silent=TRUE);
                                  dY = try(mahalanobis(Y, apply(Y,2,mean), ginv(var(Y)), inverted=TRUE), silent=TRUE);
-                                 #dX = try(mahalanobis(X, apply(Y,2,mean), var(Y)), silent=TRUE);
-                                 #dY = try(mahalanobis(Y, apply(Y,2,mean), var(Y)), silent=TRUE);
 			     } else if(method == "delta"){
 				 dX = dist(rbind(apply(Y,2,mean),X), method="manhattan") * ncol(Y)^-1 * 1000;
 				 dY = dist(rbind(apply(Y,2,mean),Y), method="manhattan")[1:nrow(Y)] * ncol(Y)^-1 * 1000;
@@ -488,7 +489,6 @@ sub signature_dist {
 }
 
 
-
 =head2 palindrome
 
  Name: palindrome   -   searches palindrome sequences
@@ -606,7 +606,7 @@ sub palindrome {
     for Actinobacteria,       5'-TTSRCCGATAATVNACATTATGTCAAGT-3'. (Hendrickson and Lawrence, 2007)
     
   Usage: 
-    @position = find_dif($sequence)
+    @position = find_dif($genome)
     
   Options:
     -type      ecoli for E.coli dif (default)
@@ -892,8 +892,16 @@ sub oligomer_counter {
 	return (@wincount);
     }elsif($length){
 	my %oligo;
-	for(my $i = 0; $i <= length($gb->{SEQ}) - $length; $i ++){
-	    $oligo{substr($gb->{SEQ}, $i, $length)} ++;
+
+	if($length == 1){
+	    $oligo{'a'} = $gb->{SEQ} =~ tr/a/a/;
+	    $oligo{'t'} = $gb->{SEQ} =~ tr/t/t/;
+	    $oligo{'g'} = $gb->{SEQ} =~ tr/g/g/;
+	    $oligo{'c'} = $gb->{SEQ} =~ tr/c/c/;
+	}else{
+	    for(my $i = 0; $i <= length($gb->{SEQ}) - $length; $i ++){
+		$oligo{substr($gb->{SEQ}, $i, $length)} ++;
+	    }
 	}
 	return %oligo;
     }else{
