@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: EPCR.pm,v 1.1.1.1 2002/04/02 20:25:44 gaou Exp $
@@ -172,13 +172,15 @@ sub togoWS{
 		$opt{db} = 'interpro';
 	    }elsif($args[0] =~ /^REACT_\d+$/){
 		$opt{db} = 'reactome';
-	    }elsif($args[0] =~ /^[a-zA-Z0-9]+_[a-zA-Z0-9]+$/){
+	    }elsif($args[0] =~ /^[a-zA-Z0-9]+_[a-zA-Z0-9*]+$/){
 		$opt{db} = 'uniprot';
 	    }else{
 		$opt{db} = 'pubmed';
 	    }
 	}
 
+	@args = readFile('http://rest.g-language.org/gbp/wild.pl?id=' . $args[0], 1) if($args[0] =~ /\*/);
+
 	my $url = 'http://togows.dbcls.jp/entry/' . $opt{db} . '/' . join(',', @args);
 	$url .= '/' . $opt{field} if (length $opt{field});
 	$url .= '.' . $opt{format} if (length $opt{format});
