@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2010 Keio University
+#     Copyright (C) 2001-2011 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Rcmd.pm,v 1.3 2002/07/30 17:40:56 gaou Exp $
@@ -85,6 +85,8 @@ sub set_mode{
     if($option eq 'tmp'){
 	$this->{cmd} = $this->{tmpcmd};
 	$this->{log} = $this->{tmplog};
+    }elsif($option eq 'silent'){
+	$this->{silent} = 1;
     }else{
 	unlink($this->{tmpcmd});
 	unlink($this->{tmplog});
@@ -171,16 +173,20 @@ sub exec{
 	    }
 	    die("Error in R, exiting...\n");
 	}elsif(/Warning/){
-	    print STDERR $_;
-	    while(<$fh2>){
+	    unless($this->{silent} == 1){
 		print STDERR $_;
-		last;
+		while(<$fh2>){
+		    print STDERR $_;
+		    last;
+		}
+		warn("Warning in R...\n");
 	    }
-	    warn("Warning in R...\n");
 	}elsif(/^Read \d+ items/){
 
 	}else{
-	    print STDERR $_;
+	    unless($this->{silent} == 1){
+		print STDERR $_;
+	    }
 	}
 
     }
